<?php

namespace Drupal\laravel_process\Plugin\Action;

use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Plugin\Action\ConfigurableActionBase;
use Drupal\eca\Service\YamlParser;
use Drupal\laravel_process\Service\Process;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Exception\ParseException;

/**
 * ECA run process.
 *
 * @Action(
 *   id = "laravel_process",
 *   label = @Translation("Laravel process"),
 *   description = @Translation("Run laravel process."),
 * )
 */
class EcaProcess extends ConfigurableActionBase {

  /**
   * The YAML parser.
   */
  protected YamlParser $yamlParser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->yamlParser = $container->get('eca.service.yaml_parser');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'command' => '',
      'directory' => '',
      'env' => '',
      'timeout' => 60,
      'token_name' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['command'] = [
      '#type' => 'textarea',
      '#required' => FALSE,
      '#title' => $this->t('Command'),
      '#description' => $this->t('The process command.'),
      '#cols' => 60,
      '#rows' => 5,
      '#default_value' => $this->configuration['command'],
    ];

    $form['directory'] = [
      '#type' => 'textfield',
      '#required' => FALSE,
      '#title' => $this->t('Working Directory'),
      '#default_value' => $this->configuration['directory'],
    ];

    $form['env'] = [
      '#type' => 'textarea',
      '#required' => FALSE,
      '#title' => $this->t('Environment variables'),
      '#description' => $this->t('The Environment variables for process with yaml format.'),
      '#cols' => 60,
      '#rows' => 5,
      '#default_value' => $this->configuration['directory'],
    ];

    $form['timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Timeout'),
      '#default_value' => $this->configuration['timeout'],
    ];

    $form['token_name'] = [
      '#required' => TRUE,
      '#type' => 'textfield',
      '#title' => $this->t('Name of results token'),
      '#default_value' => $this->configuration['token_name'],
      '#description' => $this->t('The results process will be loaded into this specified token. You can access [token:successful], [token:output], [token:errorOutput], [token:exitCode].'),
      '#eca_token_reference' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['command'] = $form_state->getValue('command');
    $this->configuration['directory'] = $form_state->getValue('directory');
    $this->configuration['env'] = $form_state->getValue('env');
    $this->configuration['timeout'] = $form_state->getValue('timeout');
    $this->configuration['token_name'] = $form_state->getValue('token_name');
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    $command = (string) $this->configuration['command'];
    $directory = (string) $this->configuration['directory'];
    $timeout = (int) $this->tokenService->getOrReplace($this->configuration['command']);
    if ($command && mb_strlen($command) > 0) {
      $command = $this->tokenService->getOrReplace($command);
    }
    if (!mb_strlen($command)) {
      return;
    }
    $env = $this->tokenService->getOrReplace($this->configuration['env']);
    if (!empty($env)) {
      try {
        $env = $this->yamlParser->parse($env);
      }
      catch (ParseException $e) {
        $this->logger->error('Tried parsing Environment variables as YAML format, but parsing failed.');
        return;
      }
    }

    $process = Process::timeout($timeout);

    if (mb_strlen($directory)) {
      $process->path($directory);
    }

    if (!empty($env)) {
      $process->env((array) $env);
    }

    $result = $process->run($command);

    $results = [
      'successful' => $result->successful(),
      'output' => $result->output(),
      'errorOutput' => $result->errorOutput(),
      'exitCode' => $result->exitCode(),
    ];

    if (!empty($this->configuration['token_name'])) {
      $this->tokenService->addTokenData($this->configuration['token_name'], $results);
    }
  }

}
